@@ -18,33 +18,34 @@ from keras_cv_attention_models.common_layers import (
     make_divisible,
     output_block,
     se_module,
+    addaptive_pooling_2d,
     PreprocessInput,
-    imagenet_decode_predictions,
     add_pre_post_process,
 )
 from keras_cv_attention_models.aotnet.aotnet import aot_stack, aot_block, deep_stem, quad_stem, tiered_stem
-from keras_cv_attention_models.botnet.botnet import RelativePositionalEmbedding, mhsa_with_relative_position_embedding
-from keras_cv_attention_models.cotnet.cotnet import cot_attention
-from keras_cv_attention_models.coat.coat import ConvPositionalEncoding, ConvRelativePositionalEncoding
 from keras_cv_attention_models.efficientnet.efficientnet_v2 import inverted_residual_block
-from keras_cv_attention_models.halonet.halonet import halo_attention
 from keras_cv_attention_models.resnest.resnest import rsoftmax, split_attention_conv2d
-from keras_cv_attention_models.volo.volo import outlook_attention, outlook_attention_simple, BiasLayer, PositionalEmbedding, ClassToken, MixupToken
-from keras_cv_attention_models.mlp_family.mlp_mixer import mlp_block, mixer_block
+from keras_cv_attention_models.mlp_family.mlp_mixer import mlp_block, mlp_mixer_block
 from keras_cv_attention_models.mlp_family.res_mlp import ChannelAffine
 from keras_cv_attention_models.mlp_family.gated_mlp import spatial_gating_block
 from keras_cv_attention_models.mlp_family.wave_mlp import phase_aware_token_mixing
-from keras_cv_attention_models.levit.levit import (
-    MultiHeadPositionalEmbedding,
-    mhsa_with_multi_head_position,
-    mhsa_with_multi_head_position_and_strides,
+from keras_cv_attention_models.volo.volo import outlook_attention, outlook_attention_simple, BiasLayer, PositionalEmbedding, ClassToken, MixupToken
+from keras_cv_attention_models.beit.beit import (
+    MultiHeadRelativePositionalEmbedding,
+    HeadInitializer,
+    PatchConv2DWithResampleWeights,
+    qkv_to_multi_head_channels_last_format,
     scaled_dot_product_attention,
 )
+from keras_cv_attention_models.botnet.botnet import RelativePositionalEmbedding, mhsa_with_relative_position_embedding
+from keras_cv_attention_models.cotnet.cotnet import cot_attention
+from keras_cv_attention_models.coat.coat import ConvPositionalEncoding, ConvRelativePositionalEncoding
+from keras_cv_attention_models.halonet.halonet import halo_attention
+from keras_cv_attention_models.convnext.convnext import global_response_normalize, add_with_layer_scale_and_drop_block
+from keras_cv_attention_models.levit.levit import MultiHeadPositionalEmbedding, mhsa_with_multi_head_position, mhsa_with_multi_head_position_and_strides
 from keras_cv_attention_models.nat.nat import MultiHeadRelativePositionalKernelBias, neighborhood_attention
 from keras_cv_attention_models.nfnets.nfnets import ScaledStandardizedConv2D, ZeroInitGain
-from keras_cv_attention_models.beit.beit import MultiHeadRelativePositionalEmbedding, HeadInitializer, PatchConv2DWithResampleWeights
 from keras_cv_attention_models.coatnet.coatnet import mhsa_with_multi_head_relative_position_embedding
-from keras_cv_attention_models.convnext.convnext import global_response_normalize, add_with_layer_scale_and_drop_block
 from keras_cv_attention_models.cmt.cmt import light_mhsa_with_multi_head_relative_position_embedding, BiasPositionalEmbedding
 from keras_cv_attention_models.uniformer.uniformer import multi_head_self_attention
 from keras_cv_attention_models.davit.davit import multi_head_self_attention_channel, window_attention
@@ -53,11 +54,14 @@ from keras_cv_attention_models.mobilevit.mobilevit import linear_self_attention
 from keras_cv_attention_models.swin_transformer_v2.swin_transformer_v2 import (
     ExpLogitScale,
     PairWiseRelativePositionalEmbedding,
+    PairWiseRelativePositionalEmbeddingGather,
     shifted_window_attention,
     WindowAttentionMask,
     window_mhsa_with_pair_wise_positional_embedding,
 )
 from keras_cv_attention_models.hornet.hornet import ComplexDense, global_local_filter, gnconv
+from keras_cv_attention_models.gpvit.gpvit import PureWeigths
+from keras_cv_attention_models.pvt.pvt import mlp_block_with_depthwise_conv
 
 CompatibleExtractPatches.__doc__ = """
 For issue https://github.com/leondgarse/keras_cv_attention_models/issues/8,
