@@ -1,6 +1,6 @@
 import os
-import tensorflow as tf
-from tensorflow import keras
+import numpy as np
+from keras_cv_attention_models.backend import layers, models, functional, image_data_format, get_file
 
 
 def reload_model_weights(
@@ -22,7 +22,7 @@ def reload_model_weights(
     if isinstance(file_hash, dict):
         # file_hash is a dict like {224: "aa", 384: "bb", 480: "cc"}
         if request_resolution == -1:
-            input_height = model.input_shape[1]
+            input_height = model.input_shape[2]  # Either channels_last or channels_first, 2 aixs has to be a shape one
             if input_height is None:  # input_shape is (None, None, 3)
                 request_resolution = max(file_hash.keys())
             else:
@@ -37,7 +37,7 @@ def reload_model_weights(
     url = pre_url.format(sub_release, model.name, pretrained)
     file_name = os.path.basename(url)
     try:
-        pretrained_model = keras.utils.get_file(file_name, url, cache_subdir="models", file_hash=file_hash)
+        pretrained_model = get_file(file_name, url, cache_subdir="models", file_hash=file_hash)
     except:
         print("[Error] will not load weights, url not found or download failed:", url)
         return None
@@ -50,7 +50,10 @@ def reload_model_weights(
 
 def load_weights_with_mismatch(model, weight_file, mismatch_class=None, force_reload_mismatch=False, request_resolution=-1, method=None):
     model.load_weights(weight_file, by_name=True, skip_mismatch=True)
-    input_height, input_width = model.input_shape[1], model.input_shape[2]
+    if image_data_format() == "channels_first":
+        input_height, input_width = model.input_shape[2], model.input_shape[3]
+    else:
+        input_height, input_width = model.input_shape[1], model.input_shape[2]
 
     # if mismatch_class is not None:
     if mismatch_class is not None and (force_reload_mismatch or request_resolution != input_height or request_resolution != input_width):
@@ -58,36 +61,37 @@ def load_weights_with_mismatch(model, weight_file, mismatch_class=None, force_re
             import h5py
 
             print(">>>> Reload mismatched weights: {} -> {}".format(request_resolution, (input_height, input_width)))
-            ff = h5py.File(weight_file, mode="r")
-            weights = ff["model_weights"]
+            with h5py.File(weight_file, mode="r") as h5_file:
+                weights = h5_file["model_weights"] if "model_weights" in h5_file else h5_file  # full model or weights only
 
-            if isinstance(mismatch_class, (list, tuple)):
-                is_mismatch_class = lambda xx: any([isinstance(ii, mm) for mm in mismatch_class])
-            else:
-                is_mismatch_class = lambda xx: isinstance(ii, mismatch_class)
+                if isinstance(mismatch_class, (list, tuple)):
+                    is_mismatch_class = lambda xx: any([isinstance(ii, mm) for mm in mismatch_class])
+                else:
+                    is_mismatch_class = lambda xx: isinstance(ii, mismatch_class)
 
-            method_kwarg = {} if method is None else {"method": method}  # None for using class default one
-            for ii in model.layers:
-                if is_mismatch_class(ii) and ii.name in weights:
-                    print(">>>> Reload layer:", ii.name)
-                    ss = weights[ii.name]
-                    # ss = {ww.decode().split("/")[-1] : tf.convert_to_tensor(ss[ww]) for ww in ss.attrs['weight_names']}
-                    ss = {ww.decode("utf8") if hasattr(ww, "decode") else ww: tf.convert_to_tensor(ss[ww]) for ww in ss.attrs["weight_names"]}
-                    ss = {kk.split("/")[-1]: vv for kk, vv in ss.items()}
-                    model.get_layer(ii.name).load_resized_weights(ss, **method_kwarg)
-            ff.close()
-
-        # print(">>>> Reload mismatched PositionalEmbedding weights: {} -> {}".format(request_resolution, input_shape[0]))
-        # bb = keras.models.load_model(pretrained_model)
-        # for ii in model.layers:
-        #     if isinstance(ii, mismatch_class):
-        #         print(">>>> Reload layer:", ii.name)
-        #         model.get_layer(ii.name).load_resized_weights(bb.get_layer(ii.name))
+                method_kwarg = {} if method is None else {"method": method}  # None for using class default one
+                for ii in model.layers:
+                    if is_mismatch_class(ii) and ii.name in weights:
+                        print(">>>> Reload layer:", ii.name)
+                        ss = weights[ii.name]
+                        # ss = {ww.decode().split("/")[-1] : tf.convert_to_tensor(ss[ww]) for ww in ss.attrs['weight_names']}
+                        ss = {ww.decode("utf8") if hasattr(ww, "decode") else ww: np.array(ss[ww]) for ww in ss.attrs["weight_names"]}
+                        ss = {kk.split("/")[-1]: vv for kk, vv in ss.items()}
+                        model.get_layer(ii.name).load_resized_weights(ss, **method_kwarg)
         except Exception as error:
             print("[Error] something went wrong in load_weights_with_mismatch:", error)
             pass
 
 
+def read_h5_weights(filepath):
+    import h5py
+    import numpy as np
+
+    with h5py.File(filepath, "r") as h5_file:
+        weights = h5_file["model_weights"] if "model_weights" in h5_file else h5_file  # full model or weights only
+        return {kk: [np.array(vv[ww]) for ww in vv.attrs["weight_names"]] for kk, vv in weights.items() if len(vv) > 0}
+
+
 """ Convert PyTorch weights """
 
 
@@ -145,7 +149,7 @@ def match_layer_names_with_torch(target_names, tail_align_dict={}, full_name_ali
     return layer_names_matched_torch
 
 
-def align_layer_names_multi_stage(target_names, tail_align_dict={}, full_name_align_dict={}, tail_split_position=2, specific_match_func=None):
+def align_layer_names_multi_stage(target_names, tail_align_dict={}, full_name_align_dict={}, tail_split_position=2, specific_match_func=None, verbose=1):
     if isinstance(tail_split_position, int):
         tail_align_dict, full_name_align_dict, tail_split_position = [tail_align_dict], [full_name_align_dict], [tail_split_position]
     full_name_align_dict = full_name_align_dict if isinstance(full_name_align_dict, (list, tuple)) else [full_name_align_dict]
@@ -154,9 +158,10 @@ def align_layer_names_multi_stage(target_names, tail_align_dict={}, full_name_al
         cur_tail = tail_align_dict[idx] if idx < len(tail_align_dict) else {}
         cur_full = full_name_align_dict[idx] if idx < len(full_name_align_dict) else {}
         cur_split = tail_split_position[idx] if idx < len(tail_split_position) else tail_split_position[-1]
-        print(">>>> tail_align_dict:", cur_tail)
-        print(">>>> full_name_align_dict:", cur_full)
-        print(">>>> tail_split_position:", cur_split)
+        if verbose > 0:
+            print(">>>> tail_align_dict:", cur_tail)
+            print(">>>> full_name_align_dict:", cur_full)
+            print(">>>> tail_split_position:", cur_split)
         target_names = match_layer_names_with_torch(target_names, cur_tail, cur_full, cur_split)
 
     if specific_match_func is not None:
@@ -164,45 +169,49 @@ def align_layer_names_multi_stage(target_names, tail_align_dict={}, full_name_al
     return target_names
 
 
-def keras_reload_stacked_state_dict(model, stacked_state_dict, layer_names_matched_torch, additional_transfer={}, save_name=None):
+def keras_reload_stacked_state_dict(model, stacked_state_dict, layer_names_matched_torch, additional_transfer={}, save_name=None, verbose=1):
     import numpy as np
 
     for kk, tf_layer_name in zip(stacked_state_dict.keys(), layer_names_matched_torch):
-        print("  torch layer name: {}, tf layer name: {}".format(kk, tf_layer_name))
+        if verbose > 0:
+            print("  torch layer name: {}, tf layer name: {}".format(kk, tf_layer_name))
         tf_layer = model.get_layer(tf_layer_name)
         tf_weights = tf_layer.get_weights()
         torch_weight = stacked_state_dict[kk]
-        print("    Before: [{}] torch: {}, tf: {}".format(kk, [ii.shape for ii in torch_weight], [ii.shape for ii in tf_weights]))
+        if verbose > 0:
+            print("    Before: [{}] torch: {}, tf: {}".format(kk, [ii.shape for ii in torch_weight], [ii.shape for ii in tf_weights]))
 
-        if isinstance(tf_layer, keras.layers.DepthwiseConv2D):  # DepthwiseConv2D is instance of Conv2D, put it first
+        if isinstance(tf_layer, layers.DepthwiseConv2D):  # DepthwiseConv2D is instance of Conv2D, put it first
             torch_weight[0] = np.transpose(torch_weight[0], (2, 3, 0, 1))
-        elif isinstance(tf_layer, keras.layers.Conv2D):
+        elif isinstance(tf_layer, layers.Conv2D):
             torch_weight[0] = np.transpose(torch_weight[0], (2, 3, 1, 0))
             if len(torch_weight) > 2:  # gain
                 torch_weight[2] = np.squeeze(torch_weight[2])
-        elif isinstance(tf_layer, keras.layers.PReLU):
+        elif isinstance(tf_layer, layers.PReLU):
             torch_weight[0] = np.expand_dims(np.expand_dims(torch_weight[0], 0), 0)
-        elif isinstance(tf_layer, keras.layers.Conv1D):
+        elif isinstance(tf_layer, layers.Conv1D):
             torch_weight[0] = np.transpose(torch_weight[0], (2, 1, 0))
-        elif isinstance(tf_layer, keras.layers.Dense):
+        elif isinstance(tf_layer, layers.Dense):
             # [Note] if it's fc layer after flatten, weights need to reshape according to NCHW --> NHWC
             torch_weight[0] = torch_weight[0].T
 
         for add_layer, add_transfer in additional_transfer.items():
             if isinstance(add_layer, str):
-                if tf_layer.name == add_layer:
+                if tf_layer.name.endswith(add_layer):
                     torch_weight = add_transfer(torch_weight)
             elif isinstance(tf_layer, add_layer):
                 torch_weight = add_transfer(torch_weight)
-        print("    After: [{}] torch: {}, tf: {}".format(kk, [ii.shape for ii in torch_weight], [ii.shape for ii in tf_weights]))
+        if verbose > 0:
+            print("    After: [{}] torch: {}, tf: {}".format(kk, [ii.shape for ii in torch_weight], [ii.shape for ii in tf_weights]))
 
         tf_layer.set_weights(torch_weight)
 
     if save_name is None:
         save_name = model.name + ".h5"
     if len(save_name) != 0:
-        print()
-        print(">>>> Save model to:", save_name)
+        if verbose > 0:
+            print()
+            print(">>>> Save model to:", save_name)
         model.save(save_name)
 
 
@@ -247,13 +256,15 @@ def keras_reload_from_torch_model(
     save_name=None,
     do_convert=True,
     do_predict=True,
+    verbose=1,
 ):
     import torch
     import numpy as np
     import tensorflow as tf
+    from tensorflow import keras
     from keras_cv_attention_models import test_images
 
-    input_shape = input_shape[:2]
+    input_shape = input_shape[:2] if keras_model is None else keras_model.input_shape[1:-1]
     if isinstance(torch_model, str):
         print(">>>> Reload Torch weight file:", torch_model)
         torch_model = torch.load(torch_model, map_location=torch.device("cpu"))
@@ -286,11 +297,13 @@ def keras_reload_from_torch_model(
     """ Convert torch weights """
     # torch_params = {kk: (np.cumproduct(vv.shape)[-1] if len(vv.shape) != 0 else 1) for kk, vv in state_dict.items() if ".num_batches_tracked" not in kk}
     stacked_state_dict = state_dict_stack_by_layer(state_dict, skip_weights=skip_weights, unstack_weights=unstack_weights)
-    print(">>>> torch_model total_parameters :", np.sum([np.sum([np.prod(jj.shape) for jj in ii]) for ii in stacked_state_dict.values()]))
+    if verbose > 0:
+        print(">>>> torch_model total_parameters :", np.sum([np.sum([np.prod(jj.shape) for jj in ii]) for ii in stacked_state_dict.values()]))
     aa = {kk: [1 if isinstance(jj, float) else jj.shape for jj in vv] for kk, vv in stacked_state_dict.items()}
-    print(">>>> Torch weights:")
-    _ = [print("  '{}': {}".format(kk, vv)) for kk, vv in aa.items()]
-    print()
+    if verbose > 0:
+        print(">>>> Torch weights:")
+        _ = [print("  '{}': {}".format(kk, vv)) for kk, vv in aa.items()]
+        print()
 
     if keras_model is None:
         return
@@ -298,18 +311,20 @@ def keras_reload_from_torch_model(
     """ Keras model weights """
     target_names = [ii.name for ii in keras_model.layers if len(ii.weights) != 0]
     aa = {keras_model.get_layer(ii).name: [jj.shape.as_list() for jj in keras_model.get_layer(ii).weights] for ii in target_names}
-    print(">>>> keras_model total_parameters :", np.sum([np.sum([np.prod(jj) for jj in ii]) for ii in aa.values()]))
-    print(">>>> Keras weights:")
-    _ = [print("  '{}': {}".format(kk, vv)) for kk, vv in aa.items()]
-    print()
+    if verbose > 0:
+        print(">>>> keras_model total_parameters :", np.sum([np.sum([int(np.prod(jj)) for jj in ii]) for ii in aa.values()]))
+        print(">>>> Keras weights:")
+        _ = [print("  '{}': {}".format(kk, vv)) for kk, vv in aa.items()]
+        print()
 
     """ Load torch weights and save h5 """
     if len(tail_align_dict) != 0 or len(full_name_align_dict) != 0 or specific_match_func is not None:
-        aligned_names = align_layer_names_multi_stage(target_names, tail_align_dict, full_name_align_dict, tail_split_position, specific_match_func)
+        aligned_names = align_layer_names_multi_stage(target_names, tail_align_dict, full_name_align_dict, tail_split_position, specific_match_func, verbose)
         aa = {keras_model.get_layer(ii).name: [jj.shape.as_list() for jj in keras_model.get_layer(ii).weights] for ii in aligned_names}
-        print(">>>> Keras weights matched torch:")
-        _ = [print("  '{}': {}".format(kk, vv)) for kk, vv in aa.items()]
-        print()
+        if verbose > 0:
+            print(">>>> Keras weights matched torch:")
+            _ = [print("  '{}': {}".format(kk, vv)) for kk, vv in aa.items()]
+            print()
     else:
         aligned_names = target_names
 
@@ -317,9 +332,11 @@ def keras_reload_from_torch_model(
         return
 
     save_name = save_name if save_name is not None else keras_model.name + "_imagenet.h5"
-    print(">>>> Keras reload torch weights:")
-    keras_reload_stacked_state_dict(keras_model, stacked_state_dict, aligned_names, additional_transfer, save_name=save_name)
-    print()
+    if verbose > 0:
+        print(">>>> Keras reload torch weights:")
+    keras_reload_stacked_state_dict(keras_model, stacked_state_dict, aligned_names, additional_transfer, save_name=save_name, verbose=verbose)
+    if verbose > 0:
+        print()
 
     """ Keras run predict """
     if do_predict:
