@@ -1,5 +1,5 @@
-import tensorflow as tf
-from tensorflow import keras
+from keras_cv_attention_models import backend
+from keras_cv_attention_models.backend import layers, functional, models, is_channels_last
 from keras_cv_attention_models.attention_layers import (
     activation_by_name,
     add_with_layer_scale_and_drop_block,
@@ -9,6 +9,7 @@ from keras_cv_attention_models.attention_layers import (
     depthwise_conv2d_no_bias,
     drop_block,
     MultiHeadPositionalEmbedding,
+    qkv_to_multi_head_channels_last_format,
     add_pre_post_process,
 )
 from keras_cv_attention_models.download_and_load import reload_model_weights
@@ -34,72 +35,88 @@ def conv_mhsa_with_multi_head_position(
     activation="gelu",
     name=None,
 ):
-    input_channel = inputs.shape[-1]
+    height_axis, width_axis, channel_aixs = (1, 2, 3) if is_channels_last() else (2, 3, 1)
+    height, width, input_channel = inputs.shape[height_axis], inputs.shape[width_axis], inputs.shape[channel_aixs]
     key_dim = key_dim if key_dim > 0 else input_channel // num_heads
-    qk_scale = float(1.0 / tf.math.sqrt(tf.cast(key_dim, "float32")))
+    # qk_scale = float(1.0 / tf.math.sqrt(tf.cast(key_dim, "float32")))
+    qk_scale = 1.0 / (float(key_dim) ** 0.5)
     out_shape = input_channel if out_shape is None else out_shape
     qk_out = num_heads * key_dim
     value_out = attn_ratio * qk_out
     value_dim = attn_ratio * key_dim
 
     if strides > 1:
-        should_cut_height, should_cut_width = inputs.shape[1] % 2, inputs.shape[2] % 2  # keep shape same with inputs after later UpSampling2D
+        should_cut_height, should_cut_width = height % 2, width % 2  # keep shape same with inputs after later UpSampling2D
         inputs = depthwise_conv2d_no_bias(inputs, use_bias=True, kernel_size=3, strides=strides, padding="same", name=name and name + "down_sample_")
         inputs = batchnorm_with_activation(inputs, activation=None, name=name and name + "down_sample_")
 
-    kv_blocks = inputs.shape[1] * inputs.shape[2]
+    kv_blocks = height * width
 
     if use_local_global_query:
-        # pool_query = keras.layers.AvgPool2D(pool_size=1, strides=2)(inputs)
-        pool_query = inputs[:, ::2, ::2]  # nn.AvgPool2d(kernel_size=1, stride=2, padding=0)
+        # pool_query = layers.AvgPool2D(pool_size=1, strides=2)(inputs)
+        # pool_query = inputs[:, ::2, ::2] if is_channels_last() else inputs[:, :, ::2, ::2] # nn.AvgPool2d(kernel_size=1, stride=2, padding=0)
+        pool_query = layers.AvgPool2D(pool_size=1, strides=2)(inputs)
         local_query = depthwise_conv2d_no_bias(inputs, use_bias=qkv_bias, kernel_size=3, strides=2, padding="same", name=name and name + "local_query_")
         pre_query = pool_query + local_query
         vv_local_strides = 2
     else:
         pre_query = inputs
         vv_local_strides = 1
-    _, query_height, query_width, _ = pre_query.shape
+    query_height, query_width = pre_query.shape[1:-1] if is_channels_last() else pre_query.shape[2:]
 
     query = conv2d_no_bias(pre_query, qk_out, use_bias=qkv_bias, kernel_size=1, name=name and name + "query_")
     query = batchnorm_with_activation(query, activation=None, name=name and name + "query_")
-    query = tf.transpose(tf.reshape(query, [-1, query_height * query_width, num_heads, key_dim]), [0, 2, 1, 3])  #  [batch, num_heads, hh * ww, key_dim]
 
     key = conv2d_no_bias(inputs, qk_out, use_bias=qkv_bias, kernel_size=1, name=name and name + "key_")
     key = batchnorm_with_activation(key, activation=None, name=name and name + "key_")
-    key = tf.transpose(tf.reshape(key, [-1, kv_blocks, num_heads, key_dim]), [0, 2, 3, 1])  #  [batch, num_heads, key_dim, hh * ww]
 
     value = conv2d_no_bias(inputs, value_out, use_bias=qkv_bias, kernel_size=1, name=name and name + "value_")
     value = batchnorm_with_activation(value, activation=None, name=name and name + "value_")
     vv_local = depthwise_conv2d_no_bias(value, use_bias=qkv_bias, kernel_size=3, strides=vv_local_strides, padding="same", name=name and name + "value_local_")
     vv_local = batchnorm_with_activation(vv_local, activation=None, name=name and name + "value_local_")
-    value = tf.transpose(tf.reshape(value, [-1, kv_blocks, num_heads, value_dim]), [0, 2, 1, 3])  #  [batch, num_heads, hh * ww, value_dim]
 
-    attention_scores = keras.layers.Lambda(lambda xx: tf.matmul(xx[0], xx[1]))([query, key]) * qk_scale  # [batch, num_heads, hh * ww, hh * ww]
+    # query = functional.transpose(
+    #     functional.reshape(query, [-1, query_height * query_width, num_heads, key_dim]), [0, 2, 1, 3]
+    # )  #  [batch, num_heads, hh * ww, key_dim]
+    # key = functional.transpose(functional.reshape(key, [-1, kv_blocks, num_heads, key_dim]), [0, 2, 3, 1])  #  [batch, num_heads, key_dim, hh * ww]
+    # value = functional.transpose(functional.reshape(value, [-1, kv_blocks, num_heads, value_dim]), [0, 2, 1, 3])  #  [batch, num_heads, hh * ww, value_dim]
+    query, key, value = qkv_to_multi_head_channels_last_format(query, key, value, num_heads=num_heads)
+
+    # attention_scores = layers.Lambda(lambda xx: functional.matmul(xx[0], xx[1]))([query, key]) * qk_scale  # [batch, num_heads, hh * ww, hh * ww]
     # print(f"{query.shape = }, {key.shape = }, {value.shape = }, {attention_scores.shape = }, {query_height = }")
+    attention_scores = (query @ key) * qk_scale
     attention_scores = MultiHeadPositionalEmbedding(query_height=query_height, name=name and name + "pos_emb")(attention_scores)
 
     if use_talking_head:
-        attention_scores = tf.transpose(attention_scores, [0, 2, 3, 1])  # [batch, hh * ww, hh * ww, num_heads]
+        # if channels_last, attention_scores [batch, num_heads, hh * ww, hh * ww] -> [batch, hh * ww, hh * ww, num_heads]
+        attention_scores = functional.transpose(attention_scores, [0, 2, 3, 1]) if is_channels_last() else attention_scores
         attention_scores = conv2d_no_bias(attention_scores, num_heads, use_bias=True, name=name and name + "talking_head_1_")
-        attention_scores = keras.layers.Softmax(axis=2, name=name and name + "attention_scores")(attention_scores)  # On previous last dimension
+        # On previous last dimension
+        attention_scores = layers.Softmax(axis=2 if is_channels_last() else -1, name=name and name + "attention_scores")(attention_scores)
         attention_scores = conv2d_no_bias(attention_scores, num_heads, use_bias=True, name=name and name + "talking_head_2_")
-        # attention_scores = keras.layers.Dropout(attn_dropout, name=name and name + "attn_drop")(attention_scores) if attn_dropout > 0 else attention_scores
-        attention_scores = tf.transpose(attention_scores, [0, 3, 1, 2])  # [batch, num_heads, hh * ww, hh * ww]
+        # if channels_last, attention_scores [batch, hh * ww, hh * ww, num_heads] -> [batch, num_heads, hh * ww, hh * ww]
+        attention_scores = functional.transpose(attention_scores, [0, 3, 1, 2]) if is_channels_last() else attention_scores
     else:
-        attention_scores = keras.layers.Softmax(axis=-1, name=name and name + "attention_scores")(attention_scores)
-        # attention_scores = keras.layers.Dropout(attn_dropout, name=name and name + "attn_drop")(attention_scores) if attn_dropout > 0 else attention_scores
+        attention_scores = layers.Softmax(axis=-1, name=name and name + "attention_scores")(attention_scores)
+        # attention_scores = layers.Dropout(attn_dropout, name=name and name + "attn_drop")(attention_scores) if attn_dropout > 0 else attention_scores
 
     # value = [batch, num_heads, hh * ww, value_dim], attention_output = [batch, num_heads, hh * ww, value_dim]
-    attention_output = keras.layers.Lambda(lambda xx: tf.matmul(xx[0], xx[1]))([attention_scores, value])
-    attention_output = tf.transpose(attention_output, perm=[0, 2, 1, 3])
-    attention_output = tf.reshape(attention_output, [-1, query_height, query_width, num_heads * value_dim])
+    # attention_output = layers.Lambda(lambda xx: functional.matmul(xx[0], xx[1]))([attention_scores, value])
+    attention_output = attention_scores @ value
+    if is_channels_last():
+        attention_output = functional.transpose(attention_output, perm=[0, 2, 1, 3])
+        attention_output = functional.reshape(attention_output, [-1, query_height, query_width, num_heads * value_dim])
+    else:
+        attention_output = functional.transpose(attention_output, perm=[0, 1, 3, 2])
+        attention_output = functional.reshape(attention_output, [-1, num_heads * value_dim, query_height, query_width])
     attention_output += vv_local
     # print(f">>>> {attention_output.shape = }, {attention_scores.shape = }")
 
     if strides > 1:
-        attention_output = keras.layers.UpSampling2D(size=(strides, strides), interpolation="bilinear")(attention_output)
+        attention_output = layers.UpSampling2D(size=(strides, strides), interpolation="bilinear")(attention_output)
         if should_cut_height > 0 or should_cut_width > 0:
-            attention_output = attention_output[:, : attention_output.shape[1] - should_cut_height, : attention_output.shape[2] - should_cut_width]
+            keep_height, keep_width = attention_output.shape[height_axis] - should_cut_height, attention_output.shape[width_axis] - should_cut_width
+            attention_output = attention_output[:, :keep_height, :keep_width] if is_channels_last() else attention_output[:, :, :keep_height, :keep_width]
 
     # [batch, hh, ww, num_heads * value_dim] * [num_heads * value_dim, out] --> [batch, hh, ww, out]
     attention_output = activation_by_name(attention_output, activation=activation, name=name)
@@ -110,7 +127,7 @@ def conv_mhsa_with_multi_head_position(
 
 
 def mlp_block_with_additional_depthwise_conv(inputs, hidden_dim, output_channel=-1, drop_rate=0, activation="gelu", name=""):
-    output_channel = output_channel if output_channel > 0 else inputs.shape[-1]
+    output_channel = output_channel if output_channel > 0 else inputs.shape[-1 if is_channels_last() else 1]
     nn = conv2d_no_bias(inputs, hidden_dim, kernel_size=1, use_bias=True, name=name + "1_")
     nn = batchnorm_with_activation(nn, activation=activation, name=name + "1_")
 
@@ -119,7 +136,7 @@ def mlp_block_with_additional_depthwise_conv(inputs, hidden_dim, output_channel=
 
     nn = conv2d_no_bias(nn, output_channel, kernel_size=1, use_bias=True, name=name + "2_")
     nn = batchnorm_with_activation(nn, activation=None, name=name + "2_")
-    nn = keras.layers.Dropout(drop_rate)(nn) if drop_rate > 0 else nn
+    nn = layers.Dropout(drop_rate)(nn) if drop_rate > 0 else nn
     return nn
 
 
@@ -154,7 +171,10 @@ def EfficientFormerV2(
     model_name="efficientformer_v2",
     kwargs=None,
 ):
-    inputs = keras.layers.Input(input_shape)
+    # Regard input_shape as force using original shape if first element is None or -1,
+    # else assume channel dimention is the one with min value in input_shape, and put it first or last regarding image_data_format
+    input_shape = backend.align_input_shape_by_image_data_format(input_shape)
+    inputs = layers.Input(input_shape)
     stem_width = stem_width if stem_width > 0 else out_channels[0]
     stem_activation = stem_activation if stem_activation is not None else activation
     nn = conv2d_no_bias(inputs, stem_width // 2, 3, strides=2, use_bias=True, padding="same", name="stem_1_")
@@ -181,25 +201,25 @@ def EfficientFormerV2(
                 strides = 2 if stack_id == 2 else 1
                 attn_out = conv_mhsa_with_multi_head_position(nn, num_heads=8, key_dim=32, strides=strides, activation=activation, name=block_name + "attn_")
                 nn = add_with_layer_scale_and_drop_block(nn, attn_out, layer_scale=layer_scale, drop_rate=block_drop_rate, name=block_name + "attn_")
-            mlp_out = mlp_block_with_additional_depthwise_conv(nn, nn.shape[-1] * cur_mlp_ratio, activation=activation, name=block_name + "mlp_")
+            mlp_out = mlp_block_with_additional_depthwise_conv(nn, out_channel * cur_mlp_ratio, activation=activation, name=block_name + "mlp_")
             nn = add_with_layer_scale_and_drop_block(nn, mlp_out, layer_scale=layer_scale, drop_rate=block_drop_rate, name=block_name + "mlp_")
             global_block_id += 1
 
     """ output """
     if num_classes > 0:
         nn = batchnorm_with_activation(nn, activation=None, name="pre_output_")
-        nn = keras.layers.GlobalAveragePooling2D()(nn)  # tf.reduce_mean(nn, axis=1)
+        nn = layers.GlobalAveragePooling2D()(nn)  # tf.reduce_mean(nn, axis=1)
         if dropout > 0 and dropout < 1:
-            nn = keras.layers.Dropout(dropout)(nn)
-        out = keras.layers.Dense(num_classes, dtype="float32", activation=classifier_activation, name="head")(nn)
+            nn = layers.Dropout(dropout)(nn)
+        out = layers.Dense(num_classes, dtype="float32", activation=classifier_activation, name="head")(nn)
 
         if use_distillation:
-            distill = keras.layers.Dense(num_classes, dtype="float32", activation=classifier_activation, name="distill_head")(nn)
+            distill = layers.Dense(num_classes, dtype="float32", activation=classifier_activation, name="distill_head")(nn)
             out = [out, distill]
     else:
         out = nn
 
-    model = keras.models.Model(inputs, out, name=model_name)
+    model = models.Model(inputs, out, name=model_name)
     add_pre_post_process(model, rescale_mode="torch")
     reload_model_weights(model, PRETRAINED_DICT, "efficientformer", pretrained, MultiHeadPositionalEmbedding)
     return model
