@@ -1,21 +1,24 @@
-import tensorflow as tf
-from tensorflow import keras
-import tensorflow.keras.backend as K
+import numpy as np
+from keras_cv_attention_models import backend
+from keras_cv_attention_models.backend import layers, models, functional, image_data_format, initializers
 
 """ Convert and replacing """
 
 
-class SAMModel(keras.models.Model):
+class SAMModel(models.Model):
     """
     Arxiv article: [Sharpness-Aware Minimization for Efficiently Improving Generalization](https://arxiv.org/pdf/2010.01412.pdf)
     Implementation by: [Keras SAM (Sharpness-Aware Minimization)](https://qiita.com/T-STAR/items/8c3afe3a116a8fc08429)
 
-    Usage is same with `keras.modeols.Model`: `model = SAMModel(inputs, outputs, rho=sam_rho, name=name)`
+    Usage is same with `keras.models.Model`: `model = SAMModel(inputs, outputs, rho=sam_rho, name=name)`
     """
 
     def __init__(self, *args, rho=0.05, **kwargs):
         super().__init__(*args, **kwargs)
+        import tensorflow as tf
+
         self.rho = tf.constant(rho, dtype=tf.float32)
+        self.tf = tf
 
     def train_step(self, data):
         if len(data) == 3:
@@ -25,14 +28,14 @@ class SAMModel(keras.models.Model):
             x, y = data
 
         # 1st step
-        with tf.GradientTape() as tape:
+        with self.tf.GradientTape() as tape:
             y_pred = self(x, training=True)
             loss = self.compiled_loss(y, y_pred, sample_weight=sample_weight, regularization_losses=self.losses)
 
         trainable_vars = self.trainable_variables
         gradients = tape.gradient(loss, trainable_vars)
 
-        norm = tf.linalg.global_norm(gradients)
+        norm = self.tf.linalg.global_norm(gradients)
         scale = self.rho / (norm + 1e-12)
         e_w_list = []
         for v, grad in zip(trainable_vars, gradients):
@@ -41,7 +44,7 @@ class SAMModel(keras.models.Model):
             e_w_list.append(e_w)
 
         # 2nd step
-        with tf.GradientTape() as tape:
+        with self.tf.GradientTape() as tape:
             y_pred_adv = self(x, training=True)
             loss_adv = self.compiled_loss(y, y_pred_adv, sample_weight=sample_weight, regularization_losses=self.losses)
         gradients_adv = tape.gradient(loss_adv, trainable_vars)
@@ -62,8 +65,8 @@ class SAMModel(keras.models.Model):
         return return_metrics
 
 
-@keras.utils.register_keras_serializable(package="model_surgery")
-class DropConnect(keras.layers.Layer):
+@backend.register_keras_serializable(package="model_surgery")
+class DropConnect(layers.Layer):
     def __init__(self, rate=0, **kwargs):
         super(DropConnect, self).__init__(**kwargs)
         self.rate = rate
@@ -72,14 +75,14 @@ class DropConnect(keras.layers.Layer):
     def build(self, input_shape):
         if self.rate > 0:
             noise_shape = [None] + [1] * (len(input_shape) - 1)  # [None, 1, 1, 1]
-            self.drop = keras.layers.Dropout(self.rate, noise_shape=noise_shape, name=self.name + "drop")
+            self.drop = layers.Dropout(self.rate, noise_shape=noise_shape, name=self.name + "drop")
         else:
             self.drop = lambda xx: xx
 
     def call(self, inputs, **kwargs):
         shortcut, deep = inputs
         deep = self.drop(deep)
-        return keras.layers.Add()([shortcut, deep])
+        return layers.Add()([shortcut, deep])
 
     def get_config(self):
         config = super(DropConnect, self).get_config()
@@ -89,6 +92,8 @@ class DropConnect(keras.layers.Layer):
 
 def add_l2_regularizer_2_model(model, weight_decay, custom_objects={}, apply_to_batch_normal=False, apply_to_bias=False):
     # https://github.com/keras-team/keras/issues/2717#issuecomment-456254176
+    from tensorflow import keras
+
     if 0:
         regularizers_type = {}
         for layer in model.layers:
@@ -101,28 +106,28 @@ def add_l2_regularizer_2_model(model, weight_decay, custom_objects={}, apply_to_
 
     for layer in model.layers:
         attrs = []
-        if isinstance(layer, keras.layers.Dense) or isinstance(layer, keras.layers.Conv2D):
+        if isinstance(layer, layers.Dense) or isinstance(layer, layers.Conv2D):
             # print(">>>> Dense or Conv2D", layer.name, "use_bias:", layer.use_bias)
             attrs = ["kernel_regularizer"]
             if apply_to_bias and layer.use_bias:
                 attrs.append("bias_regularizer")
-        elif isinstance(layer, keras.layers.DepthwiseConv2D):
+        elif isinstance(layer, layers.DepthwiseConv2D):
             # print(">>>> DepthwiseConv2D", layer.name, "use_bias:", layer.use_bias)
             attrs = ["depthwise_regularizer"]
             if apply_to_bias and layer.use_bias:
                 attrs.append("bias_regularizer")
-        elif isinstance(layer, keras.layers.SeparableConv2D):
+        elif isinstance(layer, layers.SeparableConv2D):
             # print(">>>> SeparableConv2D", layer.name, "use_bias:", layer.use_bias)
             attrs = ["pointwise_regularizer", "depthwise_regularizer"]
             if apply_to_bias and layer.use_bias:
                 attrs.append("bias_regularizer")
-        elif apply_to_batch_normal and isinstance(layer, keras.layers.BatchNormalization):
+        elif apply_to_batch_normal and isinstance(layer, layers.BatchNormalization):
             # print(">>>> BatchNormalization", layer.name, "scale:", layer.scale, ", center:", layer.center)
             if layer.center:
                 attrs.append("beta_regularizer")
             if layer.scale:
                 attrs.append("gamma_regularizer")
-        elif apply_to_batch_normal and isinstance(layer, keras.layers.PReLU):
+        elif apply_to_batch_normal and isinstance(layer, layers.PReLU):
             # print(">>>> PReLU", layer.name)
             attrs = ["alpha_regularizer"]
 
@@ -134,14 +139,15 @@ def add_l2_regularizer_2_model(model, weight_decay, custom_objects={}, apply_to_
     # reload the model so that Keras adds them to each layer's losses.
     # temp_weight_file = "tmp_weights.h5"
     # model.save_weights(temp_weight_file)
-    # out_model = keras.models.model_from_json(model.to_json(), custom_objects=custom_objects)
+    # out_model = models.model_from_json(model.to_json(), custom_objects=custom_objects)
     # out_model.load_weights(temp_weight_file, by_name=True)
     # os.remove(temp_weight_file)
     # return out_model
-    return keras.models.clone_model(model)
+    return models.clone_model(model)
 
 
 def replace_ReLU(model, target_activation="PReLU", **kwargs):
+    from tensorflow import keras
     from tensorflow.keras.layers import ReLU, PReLU, Activation
 
     def convert_ReLU(layer):
@@ -151,7 +157,7 @@ def replace_ReLU(model, target_activation="PReLU", **kwargs):
                 layer_name = layer.name.replace("_relu", "_prelu")
                 print(">>>> Convert ReLU:", layer.name, "-->", layer_name)
                 # Default initial value in mxnet and pytorch is 0.25
-                return PReLU(shared_axes=[1, 2], alpha_initializer=tf.initializers.Constant(0.25), name=layer_name, **kwargs)
+                return PReLU(shared_axes=[1, 2], alpha_initializer=initializers.Constant(0.25), name=layer_name, **kwargs)
             elif isinstance(target_activation, str):
                 layer_name = layer.name.replace("_relu", "_" + target_activation)
                 print(">>>> Convert ReLU:", layer.name, "-->", layer_name)
@@ -163,8 +169,8 @@ def replace_ReLU(model, target_activation="PReLU", **kwargs):
                 return target_activation(**kwargs)
         return layer
 
-    input_tensors = keras.layers.Input(model.input_shape[1:])
-    return keras.models.clone_model(model, input_tensors=input_tensors, clone_function=convert_ReLU)
+    input_tensors = layers.Input(model.input_shape[1:])
+    return models.clone_model(model, input_tensors=input_tensors, clone_function=convert_ReLU)
 
 
 def change_model_input_shape(model, new_input_shape):
@@ -176,7 +182,7 @@ def change_model_input_shape(model, new_input_shape):
 
     aa = json.loads(model.to_json())
     aa["config"]["layers"][0]["config"]["batch_input_shape"] = [None, *new_input_shape[:2], 3]
-    bb = tf.keras.models.model_from_json(json.dumps(aa))
+    bb = models.model_from_json(json.dumps(aa))
     temp_name = "__change_model_input_shape_temp__.h5"
     model.save_weights(temp_name)
     bb.load_weights(temp_name)
@@ -192,7 +198,7 @@ def replace_add_with_stochastic_depth(model, survivals=(1, 0.8)):
     """
     from tensorflow_addons.layers import StochasticDepth
 
-    add_layers = [ii.name for ii in model.layers if isinstance(ii, keras.layers.Add)]
+    add_layers = [ii.name for ii in model.layers if isinstance(ii, layers.Add)]
     total_adds = len(add_layers)
     if isinstance(survivals, float):
         survivals = [survivals] * total_adds
@@ -202,7 +208,7 @@ def replace_add_with_stochastic_depth(model, survivals=(1, 0.8)):
     survivals_dict = dict(zip(add_layers, survivals))
 
     def __replace_add_with_stochastic_depth__(layer):
-        if isinstance(layer, keras.layers.Add):
+        if isinstance(layer, layers.Add):
             layer_name = layer.name
             new_layer_name = layer_name.replace("_add", "_stochastic_depth")
             new_layer_name = new_layer_name.replace("add_", "stochastic_depth_")
@@ -214,8 +220,8 @@ def replace_add_with_stochastic_depth(model, survivals=(1, 0.8)):
                 return layer
         return layer
 
-    input_tensors = keras.layers.Input(model.input_shape[1:])
-    return keras.models.clone_model(model, input_tensors=input_tensors, clone_function=__replace_add_with_stochastic_depth__)
+    input_tensors = layers.Input(model.input_shape[1:])
+    return models.clone_model(model, input_tensors=input_tensors, clone_function=__replace_add_with_stochastic_depth__)
 
 
 def replace_add_with_drop_connect(model, drop_rate=(0, 0.2)):
@@ -224,7 +230,7 @@ def replace_add_with_drop_connect(model, drop_rate=(0, 0.2)):
     - [tfa.layers.StochasticDepth](https://www.tensorflow.org/addons/api_docs/python/tfa/layers/StochasticDepth)
     """
 
-    add_layers = [ii.name for ii in model.layers if isinstance(ii, keras.layers.Add)]
+    add_layers = [ii.name for ii in model.layers if isinstance(ii, layers.Add)]
     total_adds = len(add_layers)
     if isinstance(drop_rate, float):
         drop_rates = [drop_rate] * total_adds
@@ -234,7 +240,7 @@ def replace_add_with_drop_connect(model, drop_rate=(0, 0.2)):
     drop_conn_rate_dict = dict(zip(add_layers, drop_rates))
 
     def __replace_add_with_stochastic_depth__(layer):
-        if isinstance(layer, keras.layers.Add):
+        if isinstance(layer, layers.Add):
             layer_name = layer.name
             new_layer_name = layer_name.replace("_add", "_drop_conn")
             new_layer_name = new_layer_name.replace("add_", "drop_conn_")
@@ -246,8 +252,8 @@ def replace_add_with_drop_connect(model, drop_rate=(0, 0.2)):
                 return layer
         return layer
 
-    input_tensors = keras.layers.Input(model.input_shape[1:])
-    return keras.models.clone_model(model, input_tensors=input_tensors, clone_function=__replace_add_with_stochastic_depth__)
+    input_tensors = layers.Input(model.input_shape[1:])
+    return models.clone_model(model, input_tensors=input_tensors, clone_function=__replace_add_with_stochastic_depth__)
 
 
 def replace_stochastic_depth_with_add(model, drop_survival=False):
@@ -260,13 +266,13 @@ def replace_stochastic_depth_with_add(model, drop_survival=False):
             survival = layer.survival_probability
             print("Converting:", layer_name, "-->", new_layer_name, ", survival_probability:", survival)
             if drop_survival or not survival < 1:
-                return keras.layers.Add(name=new_layer_name)
+                return layers.Add(name=new_layer_name)
             else:
-                return keras.layers.Lambda(lambda xx: xx[0] + xx[1] * survival, name=new_layer_name)
+                return layers.Lambda(lambda xx: xx[0] + xx[1] * survival, name=new_layer_name)
         return layer
 
-    input_tensors = keras.layers.Input(model.input_shape[1:])
-    return keras.models.clone_model(model, input_tensors=input_tensors, clone_function=__replace_stochastic_depth_with_add__)
+    input_tensors = layers.Input(model.input_shape[1:])
+    return models.clone_model(model, input_tensors=input_tensors, clone_function=__replace_stochastic_depth_with_add__)
 
 
 def convert_to_token_label_model(model, pool_layer_id="auto"):
@@ -276,7 +282,7 @@ def convert_to_token_label_model(model, pool_layer_id="auto"):
         for header_layer_id, layer in enumerate(model.layers[::-1]):
             header_layer_id = num_total_layers - header_layer_id - 1
             print("[Search pool layer] header_layer_id = {}, layer.name = {}".format(header_layer_id, layer.name))
-            if isinstance(layer, keras.layers.GlobalAveragePooling2D):
+            if isinstance(layer, layers.GlobalAveragePooling2D):
                 break
         pool_layer_id = header_layer_id
 
@@ -287,7 +293,7 @@ def convert_to_token_label_model(model, pool_layer_id="auto"):
         aa = model.layers[header_layer_id]
         config = aa.get_config()
         config["name"] = config["name"] + "_token_label"
-        if isinstance(aa, keras.layers.LayerNormalization) and config["axis"] == [1]:
+        if isinstance(aa, layers.LayerNormalization) and config["axis"] == [1]:
             config["axis"] = [-1]
         # print(config)
         print("[Build new layer] header_layer_id = {}, layer.name = {}".format(header_layer_id, config["name"]))
@@ -296,7 +302,7 @@ def convert_to_token_label_model(model, pool_layer_id="auto"):
         bb.build(nn.shape)
         bb.set_weights(aa.get_weights())
         nn = bb(nn)
-    token_label_model = keras.models.Model(model.inputs[0], [*model.outputs, nn])
+    token_label_model = models.Model(model.inputs[0], [*model.outputs, nn])
     print("token_label_model.output_shape =", token_label_model.output_shape)
     return token_label_model
 
@@ -311,7 +317,7 @@ def get_actual_survival_probabilities(model):
 
 
 def get_actual_drop_connect_rates(model):
-    return [ii.rate for ii in model.layers if isinstance(ii, keras.layers.Dropout) or isinstance(ii, DropConnect)]
+    return [ii.rate for ii in model.layers if isinstance(ii, layers.Dropout) or isinstance(ii, DropConnect)]
 
 
 def get_pyramide_feature_layers(model, match_reg="^stack_?(\\d+).*output.*$"):
@@ -319,30 +325,57 @@ def get_pyramide_feature_layers(model, match_reg="^stack_?(\\d+).*output.*$"):
     import re
 
     dd = {}
+    pre_stack, pre_output_shape = 0, model.input_shape[1:]
     for ii in model.layers:
-        matched = re.match(match_reg, ii.name)
+        cur_name = ii.name
+        if cur_name.startswith("pre_output") and ii.output_shape[1:] == pre_output_shape:
+            cur_name = "stack_{}".format(pre_stack) + cur_name  # For Swin
+
+        matched = re.match(match_reg, cur_name)
         if matched is not None:
             cur_stack = "stack_" + matched[1] + "_output"
+            pre_stack, pre_output_shape = matched[1], ii.output_shape[1:]
             dd.update({cur_stack: ii})
 
     """ Filter those have same downsample rate """
-    ee = {str(vv.output_shape[1]): vv for kk, vv in dd.items()}
+    ee = {str(vv.output_shape[2]): vv for kk, vv in dd.items()}
     return list(ee.values())
 
 
+def align_pyramide_feature_output_by_image_data_format(pyramide_feature_layers):
+    aa = [ii.output_shape for ii in pyramide_feature_layers]
+    bb = [ii[1] for ii in aa]
+    if all([ii is None or bb[id - 1] is None or ii <= bb[id - 1] for id, ii in enumerate(bb[1:], start=1)]):
+        features_data_format = "channels_last"
+    else:
+        features_data_format = "channels_first"
+
+    if features_data_format == backend.image_data_format():
+        output_names = [ii.name for ii in pyramide_feature_layers]
+        outputs = [ii.output for ii in pyramide_feature_layers]
+    elif features_data_format == "channels_last":  # backend.image_data_format is "channels_first"
+        output_names = [ii.name + "_perm" for ii in pyramide_feature_layers]
+        outputs = [layers.Permute([3, 1, 2], name=ii.name + "_perm")(ii.output) for ii in pyramide_feature_layers]
+    else:  # features_data_format is "channels_first" and backend.image_data_format is "channels_last". Should not happen
+        output_names = [ii.name + "_perm" for ii in pyramide_feature_layers]
+        outputs = [layers.Permute([2, 3, 1], name=ii.name + "_perm")(ii.output) for ii in pyramide_feature_layers]
+    return output_names, outputs
+
+
 def get_global_avg_pool_layer_id(model):
     """Search GlobalAveragePooling2D layer id"""
     num_total_layers = len(model.layers)
     for header_layer_id, layer in enumerate(model.layers[::-1]):
         header_layer_id = num_total_layers - header_layer_id - 1
         print("[Search pool layer] header_layer_id = {}, layer.name = {}".format(header_layer_id, layer.name))
-        if isinstance(layer, keras.layers.GlobalAveragePooling2D):
+        if isinstance(layer, layers.GlobalAveragePooling2D):
             break
     return header_layer_id
 
 
 def get_flops(model):
     # https://github.com/tensorflow/tensorflow/issues/32809#issuecomment-849439287
+    import tensorflow as tf
     from tensorflow.python.profiler import model_analyzer, option_builder
 
     input_signature = [tf.TensorSpec(shape=(1, *ii.shape[1:]), dtype=ii.dtype, name=ii.name) for ii in model.inputs]
@@ -365,13 +398,15 @@ def print_model_params_count(model):
 
 
 def convert_to_mixed_float16(model, convert_batch_norm=False, policy_name="mixed_float16"):
-    policy = keras.mixed_precision.Policy(policy_name)
-    policy_config = keras.utils.serialize_keras_object(policy)
+    from tensorflow import keras
     from tensorflow.keras.layers import InputLayer, Activation
     from tensorflow.keras.activations import linear
 
+    policy = keras.mixed_precision.Policy(policy_name)
+    policy_config = keras.utils.serialize_keras_object(policy)
+
     def do_convert_to_mixed_float16(layer):
-        if not convert_batch_norm and isinstance(layer, keras.layers.BatchNormalization):
+        if not convert_batch_norm and isinstance(layer, layers.BatchNormalization):
             return layer
         if not isinstance(layer, InputLayer) and not (isinstance(layer, Activation) and layer.activation == linear):
             aa = layer.get_config()
@@ -382,8 +417,8 @@ def convert_to_mixed_float16(model, convert_batch_norm=False, policy_name="mixed
             return bb
         return layer
 
-    input_tensors = keras.layers.Input(model.input_shape[1:])
-    return keras.models.clone_model(model, input_tensors=input_tensors, clone_function=do_convert_to_mixed_float16)
+    input_tensors = layers.Input(model.input_shape[1:])
+    return models.clone_model(model, input_tensors=input_tensors, clone_function=do_convert_to_mixed_float16)
 
 
 def convert_mixed_float16_to_float32(model):
@@ -400,18 +435,18 @@ def convert_mixed_float16_to_float32(model):
             return bb
         return layer
 
-    input_tensors = keras.layers.Input(model.input_shape[1:])
-    return keras.models.clone_model(model, input_tensors=input_tensors, clone_function=do_convert_to_mixed_float16)
+    input_tensors = layers.Input(model.input_shape[1:])
+    return models.clone_model(model, input_tensors=input_tensors, clone_function=do_convert_to_mixed_float16)
 
 
 def fuse_conv_bn(conv_layer, bn_layer):
     # BatchNormalization returns: gamma * (batch - self.moving_mean) / sqrt(self.moving_var + epsilon) + beta
     # --> conv_w_new = gamma * conv_w / np.sqrt(var + epsilon)
     # --> conv_b_new = gamma * (conv_b - mean) / sqrt(var + epsilon) + beta
-    batch_std = tf.sqrt(bn_layer.moving_variance + bn_layer.epsilon)
-    if isinstance(conv_layer, keras.layers.DepthwiseConv2D):
-        ww = tf.transpose(conv_layer.depthwise_kernel, [0, 1, 3, 2]) * bn_layer.gamma / batch_std
-        ww = tf.transpose(ww, [0, 1, 3, 2])
+    batch_std = functional.sqrt(bn_layer.moving_variance + bn_layer.epsilon)
+    if isinstance(conv_layer, layers.DepthwiseConv2D):
+        ww = functional.transpose(conv_layer.depthwise_kernel, [0, 1, 3, 2]) * bn_layer.gamma / batch_std
+        ww = functional.transpose(ww, [0, 1, 3, 2])
     else:
         ww = conv_layer.kernel * bn_layer.gamma / batch_std
 
@@ -478,7 +513,7 @@ def convert_to_fused_conv_bn_model(model):
 
         layers.append(layer)
     model_config["config"]["layers"] = layers
-    new_model = keras.models.model_from_json(json.dumps(model_config))
+    new_model = models.model_from_json(json.dumps(model_config))
 
     """ New model set layer weights by layer names """
     for layer in new_model.layers:
@@ -499,13 +534,15 @@ def convert_to_fused_conv_bn_model(model):
 """ TFLite """
 
 
-@keras.utils.register_keras_serializable(package="model_surgery")
-class SplitConv2D(keras.layers.Conv2D):
+@backend.register_keras_serializable(package="model_surgery")
+class SplitConv2D(layers.Conv2D):
     def __init__(self, **kwargs):
         super().__init__(**kwargs)
-        self.super_class = keras.layers.Conv2D
+        self.super_class = layers.Conv2D
 
     def build(self, input_shape):
+        import tensorflow as tf
+
         cc = self.get_config().copy()
         cc.update({"groups": 1, "filters": self.filters // self.groups})
         grouped_input_shape = (*input_shape[:-1], input_shape[-1] // self.groups)
@@ -519,10 +556,10 @@ class SplitConv2D(keras.layers.Conv2D):
             self.convs.append(conv)
 
     def call(self, inputs, **kwargs):
-        return tf.concat([conv(ii) for conv, ii in zip(self.convs, tf.split(inputs, self.groups, axis=-1))], axis=-1)
+        return functional.concat([conv(ii) for conv, ii in zip(self.convs, functional.split(inputs, self.groups, axis=-1))], axis=-1)
 
 
-@keras.utils.register_keras_serializable(package="model_surgery")
+@backend.register_keras_serializable(package="model_surgery")
 class SplitScaledStandardizedConv2D(SplitConv2D):
     def __init__(self, gamma=1.0, eps=1e-5, **kwargs):
         from keras_cv_attention_models import attention_layers
@@ -538,21 +575,19 @@ class SplitScaledStandardizedConv2D(SplitConv2D):
 
 
 def convert_groups_conv2d_2_split_conv2d(model):
-    from tensorflow.keras.layers import Conv2D
-
     def __convert_groups_conv2d_2_split_conv2d__(layer):
-        if isinstance(layer, Conv2D) and not isinstance(layer, SplitConv2D) and layer.groups != 1:
+        if isinstance(layer, layers.Conv2D) and not isinstance(layer, SplitConv2D) and layer.groups != 1:
             aa = layer.get_config()
             # Check if ScaledStandardizedConv2D or typical Conv2D
             bb = SplitScaledStandardizedConv2D.from_config(aa) if hasattr(layer, "gain") else SplitConv2D.from_config(aa)
             # bb.build(layer.input_shape)   # looks like build not working [ ??? ]
-            bb(tf.ones([1, *layer.input_shape[1:]]))
-            wws = tf.split(layer.get_weights()[0], bb.groups, axis=-1)
+            bb(initializers.ones([1, *layer.input_shape[1:]]))
+            wws = functional.split(layer.get_weights()[0], bb.groups, axis=-1)
             if bb.use_bias:
-                bbs = tf.split(layer.get_weights()[1], bb.groups, axis=-1)
+                bbs = functional.split(layer.get_weights()[1], bb.groups, axis=-1)
             if hasattr(layer, "gain"):
                 # ScaledStandardizedConv2D with gain from NFNets
-                ggs = tf.split(layer.get_weights()[-1], bb.groups, axis=-1)
+                ggs = functional.split(layer.get_weights()[-1], bb.groups, axis=-1)
             for id in range(bb.groups):
                 sub_weights = [wws[id].numpy()]
                 if bb.use_bias:
@@ -563,16 +598,16 @@ def convert_groups_conv2d_2_split_conv2d(model):
             return bb
         return layer
 
-    input_tensors = keras.layers.Input(model.input_shape[1:])
-    return keras.models.clone_model(model, input_tensors=input_tensors, clone_function=__convert_groups_conv2d_2_split_conv2d__)
+    input_tensors = layers.Input(model.input_shape[1:])
+    return models.clone_model(model, input_tensors=input_tensors, clone_function=__convert_groups_conv2d_2_split_conv2d__)
 
 
 def convert_gelu_and_extract_patches_for_tflite(model):
     from keras_cv_attention_models import attention_layers
 
     def __convert_gelu_and_extract_patches_for_tflite__(layer):
-        if isinstance(layer, keras.layers.Activation) and layer.activation.__name__ == "gelu":
-            return keras.layers.Lambda(lambda xx: tf.nn.gelu(xx, approximate=True))
+        if isinstance(layer, layers.Activation) and layer.activation.__name__ == "gelu":
+            return layers.Lambda(lambda xx: functional.gelu(xx, approximate=True))
         elif isinstance(layer, attention_layers.CompatibleExtractPatches):
             aa = layer.get_config()
             aa.update({"force_conv": True})
@@ -581,8 +616,8 @@ def convert_gelu_and_extract_patches_for_tflite(model):
             return bb
         return layer
 
-    input_tensors = keras.layers.Input(model.input_shape[1:])
-    return keras.models.clone_model(model, input_tensors=input_tensors, clone_function=__convert_gelu_and_extract_patches_for_tflite__)
+    input_tensors = layers.Input(model.input_shape[1:])
+    return models.clone_model(model, input_tensors=input_tensors, clone_function=__convert_gelu_and_extract_patches_for_tflite__)
 
 
 def prepare_for_tflite(model):
