@@ -1,6 +1,8 @@
-import tensorflow as tf
-from tensorflow import keras
+import numpy as np
+from keras_cv_attention_models import backend
+from keras_cv_attention_models.backend import layers, models, functional, initializers, image_data_format
 from keras_cv_attention_models.attention_layers import (
+    add_with_layer_scale_and_drop_block,
     ChannelAffine,
     CompatibleExtractPatches,
     conv2d_no_bias,
@@ -20,8 +22,8 @@ PRETRAINED_DICT = {
 }
 
 
-@tf.keras.utils.register_keras_serializable(package="kecam/nat")
-class MultiHeadRelativePositionalKernelBias(tf.keras.layers.Layer):
+@backend.register_keras_serializable(package="kecam/nat")
+class MultiHeadRelativePositionalKernelBias(layers.Layer):
     def __init__(self, input_height=-1, is_heads_first=False, **kwargs):
         super().__init__(**kwargs)
         self.input_height, self.is_heads_first = input_height, is_heads_first
@@ -30,31 +32,36 @@ class MultiHeadRelativePositionalKernelBias(tf.keras.layers.Layer):
         # input (is_heads_first=False): `[batch, height * width, num_heads, ..., size * size]`
         # input (is_heads_first=True): `[batch, num_heads, height * width, ..., size * size]`
         blocks, num_heads = (input_shape[2], input_shape[1]) if self.is_heads_first else (input_shape[1], input_shape[2])
-        size = int(tf.math.sqrt(float(input_shape[-1])))
-        height = self.input_height if self.input_height > 0 else int(tf.math.sqrt(float(blocks)))
+        size = int(np.sqrt(float(input_shape[-1])))
+        height = self.input_height if self.input_height > 0 else int(np.sqrt(float(blocks)))
         width = blocks // height
         pos_size = 2 * size - 1
-        initializer = tf.initializers.truncated_normal(stddev=0.02)
+        initializer = initializers.truncated_normal(stddev=0.02)
         self.pos_bias = self.add_weight(name="positional_embedding", shape=(num_heads, pos_size * pos_size), initializer=initializer, trainable=True)
 
-        idx_hh, idx_ww = tf.range(0, size), tf.range(0, size)
-        coords = tf.reshape(tf.expand_dims(idx_hh, -1) * pos_size + idx_ww, [-1])
-        bias_hh = tf.concat([idx_hh[: size // 2], tf.repeat(idx_hh[size // 2], height - size + 1), idx_hh[size // 2 + 1 :]], axis=-1)
-        bias_ww = tf.concat([idx_ww[: size // 2], tf.repeat(idx_ww[size // 2], width - size + 1), idx_ww[size // 2 + 1 :]], axis=-1)
-        bias_hw = tf.expand_dims(bias_hh, -1) * pos_size + bias_ww
-        bias_coords = tf.expand_dims(bias_hw, -1) + coords
-        bias_coords = tf.reshape(bias_coords, [-1, size**2])[::-1]  # torch.flip(bias_coords, [0])
+        idx_hh, idx_ww = np.arange(0, size), np.arange(0, size)
+        coords = np.reshape(np.expand_dims(idx_hh, -1) * pos_size + idx_ww, [-1]).astype("int64")
+        bias_hh = np.concatenate([idx_hh[: size // 2], np.repeat(idx_hh[size // 2], height - size + 1), idx_hh[size // 2 + 1 :]], axis=-1)
+        bias_ww = np.concatenate([idx_ww[: size // 2], np.repeat(idx_ww[size // 2], width - size + 1), idx_ww[size // 2 + 1 :]], axis=-1)
+        bias_hw = np.expand_dims(bias_hh, -1) * pos_size + bias_ww
+        bias_coords = np.expand_dims(bias_hw, -1) + coords
+        bias_coords = np.reshape(bias_coords, [-1, size**2])[::-1]  # torch.flip(bias_coords, [0])
 
         bias_coords_shape = [bias_coords.shape[0]] + [1] * (len(input_shape) - 4) + [bias_coords.shape[1]]
-        self.bias_coords = tf.reshape(bias_coords, bias_coords_shape)  # [height * width, 1 * n, size * size]
+        bias_coords = np.reshape(bias_coords, bias_coords_shape)  # [height * width, 1 * n, size * size]
+        if hasattr(self, "register_buffer"):  # PyTorch
+            self.register_buffer("bias_coords", functional.convert_to_tensor(bias_coords, dtype="int64"), persistent=False)
+        else:
+            self.bias_coords = functional.convert_to_tensor(bias_coords, dtype="int64")
+
         if not self.is_heads_first:
             self.transpose_perm = [1, 0] + list(range(2, len(input_shape) - 1))  # transpose [num_heads, height * width] -> [height * width, num_heads]
 
     def call(self, inputs):
         if self.is_heads_first:
-            return inputs + tf.gather(self.pos_bias, self.bias_coords, axis=-1)
+            return inputs + functional.gather(self.pos_bias, self.bias_coords, axis=-1)
         else:
-            return inputs + tf.transpose(tf.gather(self.pos_bias, self.bias_coords, axis=-1), self.transpose_perm)
+            return inputs + functional.transpose(functional.gather(self.pos_bias, self.bias_coords, axis=-1), self.transpose_perm)
 
     def get_config(self):
         base_config = super().get_config()
@@ -73,37 +80,39 @@ def neighborhood_attention(
 
     should_pad_hh, should_pad_ww = max(0, kernel_size - hh), max(0, kernel_size - ww)
     if should_pad_hh or should_pad_ww:
-        inputs = tf.pad(inputs, [[0, 0], [0, should_pad_hh], [0, should_pad_ww], [0, 0]])
+        inputs = functional.pad(inputs, [[0, 0], [0, should_pad_hh], [0, should_pad_ww], [0, 0]])
         _, hh, ww, cc = inputs.shape
 
-    qkv = keras.layers.Dense(qkv_out * 3, use_bias=qkv_bias, name=name and name + "qkv")(inputs)
-    query, key_value = tf.split(qkv, [qkv_out, qkv_out * 2], axis=-1)  # Matching weights from PyTorch
-    query = tf.expand_dims(tf.reshape(query, [-1, hh * ww, num_heads, key_dim]), -2)  # [batch, hh * ww, num_heads, 1, key_dim]
+    qkv = layers.Dense(qkv_out * 3, use_bias=qkv_bias, name=name and name + "qkv")(inputs)
+    query, key_value = functional.split(qkv, [qkv_out, qkv_out * 2], axis=-1)  # Matching weights from PyTorch
+    query = functional.expand_dims(functional.reshape(query, [-1, hh * ww, num_heads, key_dim]), -2)  # [batch, hh * ww, num_heads, 1, key_dim]
 
     # key_value: [batch, height // kernel_size, width // kernel_size, kernel_size, kernel_size, key + value]
-    key_value = CompatibleExtractPatches(sizes=kernel_size, strides=1, padding="VALID", compressed=False)(key_value)
+    kv = CompatibleExtractPatches(sizes=kernel_size, strides=1, padding="VALID", compressed=False)(key_value)
     padded = (kernel_size - 1) // 2
     # torch.pad 'replicate'
-    key_value = tf.concat([tf.repeat(key_value[:, :1], padded, axis=1), key_value, tf.repeat(key_value[:, -1:], padded, axis=1)], axis=1)
-    key_value = tf.concat([tf.repeat(key_value[:, :, :1], padded, axis=2), key_value, tf.repeat(key_value[:, :, -1:], padded, axis=2)], axis=2)
+    kv = functional.concat([functional.repeat(kv[:, :1], padded, axis=1), kv, functional.repeat(kv[:, -1:], padded, axis=1)], axis=1)
+    key_value = functional.concat([functional.repeat(kv[:, :, :1], padded, axis=2), kv, functional.repeat(kv[:, :, -1:], padded, axis=2)], axis=2)
 
-    key_value = tf.reshape(key_value, [-1, kernel_size * kernel_size, key_value.shape[-1]])
-    key, value = tf.split(key_value, 2, axis=-1)  # [batch * block_height * block_width, kernel_size * kernel_size, key_dim]
-    key = tf.transpose(tf.reshape(key, [-1, key.shape[1], num_heads, key_dim]), [0, 2, 3, 1])  # [batch * hh*ww, num_heads, key_dim, kernel_size * kernel_size]
-    key = tf.reshape(key, [-1, hh * ww, num_heads, key_dim, kernel_size * kernel_size])  # [batch, hh*ww, num_heads, key_dim, kernel_size * kernel_size]
-    value = tf.transpose(tf.reshape(value, [-1, value.shape[1], num_heads, key_dim]), [0, 2, 1, 3])
-    value = tf.reshape(value, [-1, hh * ww, num_heads, kernel_size * kernel_size, key_dim])  # [batch, hh*ww, num_heads, kernel_size * kernel_size, key_dim]
+    key_value = functional.reshape(key_value, [-1, kernel_size * kernel_size, key_value.shape[-1]])
+    key, value = functional.split(key_value, 2, axis=-1)  # [batch * block_height * block_width, K * K, key_dim]
+    key = functional.transpose(functional.reshape(key, [-1, key.shape[1], num_heads, key_dim]), [0, 2, 3, 1])  # [batch * hh*ww, num_heads, key_dim, K * K]
+    key = functional.reshape(key, [-1, hh * ww, num_heads, key_dim, kernel_size * kernel_size])  # [batch, hh*ww, num_heads, key_dim, K * K]
+    value = functional.transpose(functional.reshape(value, [-1, value.shape[1], num_heads, key_dim]), [0, 2, 1, 3])
+    value = functional.reshape(value, [-1, hh * ww, num_heads, kernel_size * kernel_size, key_dim])  # [batch, hh*ww, num_heads, K * K, key_dim]
     # print(f">>>> {query.shape = }, {key.shape = }, {value.shape = }")
 
     # [batch, hh * ww, num_heads, 1, kernel_size * kernel_size]
-    attention_scores = keras.layers.Lambda(lambda xx: tf.matmul(xx[0], xx[1]))([query, key]) * qk_scale
+    # attention_scores = layers.Lambda(lambda xx: tf.matmul(xx[0], xx[1]))([query, key]) * qk_scale
+    attention_scores = (query @ key) * qk_scale
     attention_scores = MultiHeadRelativePositionalKernelBias(input_height=hh, name=name and name + "pos")(attention_scores)
-    attention_scores = keras.layers.Softmax(axis=-1, name=name and name + "attention_scores")(attention_scores)
-    attention_scores = keras.layers.Dropout(attn_dropout, name=name and name + "attn_drop")(attention_scores) if attn_dropout > 0 else attention_scores
+    attention_scores = layers.Softmax(axis=-1, name=name and name + "attention_scores")(attention_scores)
+    attention_scores = layers.Dropout(attn_dropout, name=name and name + "attn_drop")(attention_scores) if attn_dropout > 0 else attention_scores
 
     # attention_output = [batch, block_height * block_width, num_heads, 1, key_dim]
-    attention_output = keras.layers.Lambda(lambda xx: tf.matmul(xx[0], xx[1]))([attention_scores, value])
-    attention_output = tf.reshape(attention_output, [-1, hh, ww, num_heads * key_dim])
+    # attention_output = layers.Lambda(lambda xx: tf.matmul(xx[0], xx[1]))([attention_scores, value])
+    attention_output = attention_scores @ value
+    attention_output = functional.reshape(attention_output, [-1, hh, ww, num_heads * key_dim])
     # print(f">>>> {attention_output.shape = }, {attention_scores.shape = }")
 
     if should_pad_hh or should_pad_ww:
@@ -111,8 +120,8 @@ def neighborhood_attention(
 
     if out_weight:
         # [batch, hh, ww, num_heads * key_dim] * [num_heads * key_dim, out] --> [batch, hh, ww, out]
-        attention_output = keras.layers.Dense(out_shape, use_bias=out_bias, name=name and name + "output")(attention_output)
-    attention_output = keras.layers.Dropout(output_dropout, name=name and name + "out_drop")(attention_output) if output_dropout > 0 else attention_output
+        attention_output = layers.Dense(out_shape, use_bias=out_bias, name=name and name + "output")(attention_output)
+    attention_output = layers.Dropout(output_dropout, name=name and name + "out_drop")(attention_output) if output_dropout > 0 else attention_output
     return attention_output
 
 
@@ -121,15 +130,11 @@ def nat_block(inputs, attn_kernel_size=7, num_heads=4, mlp_ratio=4, mlp_drop_rat
 
     attn = layer_norm(inputs, name=name + "attn_")
     attn = neighborhood_attention(attn, attn_kernel_size, num_heads, attn_dropout=attn_drop_rate, name=name + "attn_")
-    attn = ChannelAffine(use_bias=False, weight_init_value=layer_scale, name=name + "1_gamma")(attn) if layer_scale >= 0 else attn
-    attn = drop_block(attn, drop_rate=drop_rate, name=name + "attn_")
-    attn_out = keras.layers.Add(name=name + "attn_out")([inputs, attn])
+    attn_out = add_with_layer_scale_and_drop_block(inputs, attn, layer_scale=layer_scale, drop_rate=drop_rate, name=name + "1_")
 
     mlp = layer_norm(attn_out, name=name + "mlp_")
-    mlp = mlp_block(mlp, int(input_channel * mlp_ratio), drop_rate=mlp_drop_rate, use_conv=False, activation="gelu", name=name + "mlp_")
-    mlp = ChannelAffine(use_bias=False, weight_init_value=layer_scale, name=name + "2_gamma")(mlp) if layer_scale >= 0 else mlp
-    mlp = drop_block(mlp, drop_rate=drop_rate, name=name + "mlp_")
-    return keras.layers.Add(name=name + "output")([attn_out, mlp])
+    mlp = mlp_block(mlp, int(input_channel * mlp_ratio), drop_rate=mlp_drop_rate, activation="gelu", name=name + "mlp_")
+    return add_with_layer_scale_and_drop_block(attn_out, mlp, layer_scale=layer_scale, drop_rate=drop_rate, name=name + "2_")
 
 
 def NAT(
@@ -150,7 +155,10 @@ def NAT(
     kwargs=None,
 ):
     """ConvTokenizer stem"""
-    inputs = keras.layers.Input(input_shape)
+    # Regard input_shape as force using original shape if len(input_shape) == 4,
+    # else assume channel dimention is the one with min value in input_shape, and put it first or last regarding image_data_format
+    input_shape = backend.align_input_shape_by_image_data_format(input_shape)
+    inputs = layers.Input(input_shape)
     stem_width = stem_width if stem_width > 0 else out_channels[0]
     nn = conv2d_no_bias(inputs, stem_width // 2, kernel_size=3, strides=2, use_bias=True, padding="SAME", name="stem_1_")
     nn = conv2d_no_bias(nn, stem_width, kernel_size=3, strides=2, use_bias=True, padding="SAME", name="stem_2_")
@@ -173,7 +181,7 @@ def NAT(
     nn = layer_norm(nn, name="pre_output_")
 
     nn = output_block(nn, num_classes=num_classes, drop_rate=dropout, classifier_activation=classifier_activation)
-    model = keras.models.Model(inputs, nn, name=model_name)
+    model = models.Model(inputs, nn, name=model_name)
     add_pre_post_process(model, rescale_mode="torch")
     reload_model_weights(model, PRETRAINED_DICT, "nat", pretrained)
     return model
