@@ -1,5 +1,5 @@
-import tensorflow as tf
-from tensorflow import keras
+from keras_cv_attention_models import backend
+from keras_cv_attention_models.backend import layers, functional, models, initializers, image_data_format
 from keras_cv_attention_models.attention_layers import (
     BiasLayer,
     ChannelAffine,
@@ -40,18 +40,19 @@ def conv_dw_pw_block(inputs, filters, kernel_size=1, strides=1, use_depthwise_co
 
 
 def csp_block(inputs, expansion=0.5, use_shortcut=True, activation="swish", name=""):
-    input_channels = inputs.shape[-1]
+    input_channels = inputs.shape[-1 if image_data_format() == "channels_last" else 1]
     nn = conv_dw_pw_block(inputs, int(input_channels * expansion), activation=activation, name=name + "1_")
     nn = conv_dw_pw_block(nn, input_channels, kernel_size=3, strides=1, activation=activation, name=name + "2_")
     if use_shortcut:
-        nn = keras.layers.Add()([inputs, nn])
+        nn = layers.Add()([inputs, nn])
     return nn
 
 
 def csp_stack(
     inputs, depth, out_channels=-1, expansion=0.5, use_shortcut=True, use_pre=False, use_post=True, use_shortcut_bn=True, activation="swish", name=""
 ):
-    out_channels = inputs.shape[-1] if out_channels == -1 else out_channels
+    channel_axis = -1 if image_data_format() == "channels_last" else 1
+    out_channels = inputs.shape[channel_axis] if out_channels == -1 else out_channels
     hidden_channels = int(out_channels * expansion)
     if use_pre:
         inputs = conv_dw_pw_block(inputs, hidden_channels, kernel_size=1, activation=activation, name=name + "pre_")
@@ -67,7 +68,7 @@ def csp_stack(
     if use_post:
         deep = conv_dw_pw_block(deep, hidden_channels, kernel_size=1, activation=activation, name=name + "deep_post_")
 
-    out = tf.concat([deep, short], axis=-1)
+    out = functional.concat([deep, short], axis=channel_axis)
     if not use_shortcut_bn:
         out = batchnorm_with_activation(out, activation=activation, epsilon=BATCH_NORM_EPSILON, momentum=BATCH_NORM_MOMENTUM, name=name + "concat_")
 
@@ -76,7 +77,8 @@ def csp_stack(
 
 
 def res_spatial_pyramid_pooling(inputs, depth, expansion=0.5, pool_sizes=(5, 9, 13), use_shortcut_bn=True, activation="swish", name=""):
-    input_channels = inputs.shape[-1]
+    channel_axis = -1 if image_data_format() == "channels_last" else 1
+    input_channels = inputs.shape[channel_axis]
     hidden_channels = int(input_channels * expansion)
     if use_shortcut_bn:
         short = conv_dw_pw_block(inputs, hidden_channels, kernel_size=1, activation=activation, name=name + "short_")
@@ -86,13 +88,13 @@ def res_spatial_pyramid_pooling(inputs, depth, expansion=0.5, pool_sizes=(5, 9,
     deep = conv_dw_pw_block(inputs, hidden_channels, kernel_size=1, activation=activation, name=name + "pre_1_")
     deep = conv_dw_pw_block(deep, hidden_channels, kernel_size=3, activation=activation, name=name + "pre_2_")
     deep = conv_dw_pw_block(deep, hidden_channels, kernel_size=1, activation=activation, name=name + "pre_3_")
-    pp = [keras.layers.MaxPooling2D(pool_size=ii, strides=1, padding="SAME")(deep) for ii in pool_sizes]
-    deep = tf.concat([deep, *pp][::-1], axis=-1)  # yolor_csp.cfg, SSP concat layers=-1,-3,-5,-6
+    pp = [layers.MaxPool2D(pool_size=ii, strides=1, padding="SAME")(deep) for ii in pool_sizes]
+    deep = functional.concat([deep, *pp][::-1], axis=channel_axis)  # yolor_csp.cfg, SSP concat layers=-1,-3,-5,-6
     for id in range(depth - 1):  # First one is `pre`
         deep = conv_dw_pw_block(deep, hidden_channels, kernel_size=1, activation=activation, name=name + "post_{}_".format(id * 2 + 1))
         deep = conv_dw_pw_block(deep, hidden_channels, kernel_size=3, activation=activation, name=name + "post_{}_".format(id * 2 + 2))
 
-    out = tf.concat([deep, short], axis=-1)
+    out = functional.concat([deep, short], axis=channel_axis)
     if not use_shortcut_bn:
         out = batchnorm_with_activation(out, activation=activation, epsilon=BATCH_NORM_EPSILON, momentum=BATCH_NORM_MOMENTUM, name=name + "concat_")
     out = conv_dw_pw_block(out, hidden_channels, kernel_size=1, activation=activation, name=name + "output_")
@@ -100,29 +102,32 @@ def res_spatial_pyramid_pooling(inputs, depth, expansion=0.5, pool_sizes=(5, 9,
 
 
 def focus_stem(inputs, filters, kernel_size=3, strides=1, padding="valid", activation="swish", name=""):
+    is_channels_last = image_data_format() == "channels_last"
+    channel_axis = -1 if is_channels_last else 1
     if padding.lower() == "same":  # Handling odd input_shape
-        inputs = tf.pad(inputs, [[0, 0], [0, 1], [0, 1], [0, 0]])
-        patch_top_left = inputs[:, :-1:2, :-1:2]
-        patch_top_right = inputs[:, :-1:2, 1::2]
-        patch_bottom_left = inputs[:, 1::2, :-1:2]
-        patch_bottom_right = inputs[:, 1::2, 1::2]
+        inputs = functional.pad(inputs, [[0, 0], [0, 1], [0, 1], [0, 0]] if is_channels_last else [[0, 0], [0, 0], [0, 1], [0, 1]])
+        patch_top_left = inputs[:, :-1:2, :-1:2] if is_channels_last else inputs[:, :, :-1:2, :-1:2]
+        patch_top_right = inputs[:, :-1:2, 1::2] if is_channels_last else inputs[:, :, :-1:2, 1::2]
+        patch_bottom_left = inputs[:, 1::2, :-1:2] if image_data_format() == "channels_last" else inputs[:, :, 1::2, :-1:2]
+        patch_bottom_right = inputs[:, 1::2, 1::2] if image_data_format() == "channels_last" else inputs[:, :, 1::2, 1::2]
     else:
-        patch_top_left = inputs[:, ::2, ::2]
-        patch_top_right = inputs[:, ::2, 1::2]
-        patch_bottom_left = inputs[:, 1::2, ::2]
-        patch_bottom_right = inputs[:, 1::2, 1::2]
-    nn = tf.concat([patch_top_left, patch_bottom_left, patch_top_right, patch_bottom_right], axis=-1)
+        patch_top_left = inputs[:, ::2, ::2] if is_channels_last else inputs[:, :, ::2, ::2]
+        patch_top_right = inputs[:, ::2, 1::2] if is_channels_last else inputs[:, :, ::2, 1::2]
+        patch_bottom_left = inputs[:, 1::2, ::2] if is_channels_last else inputs[:, :, 1::2, ::2]
+        patch_bottom_right = inputs[:, 1::2, 1::2] if is_channels_last else inputs[:, :, 1::2, 1::2]
+    nn = functional.concat([patch_top_left, patch_bottom_left, patch_top_right, patch_bottom_right], axis=channel_axis)
     nn = conv_dw_pw_block(nn, filters, kernel_size=kernel_size, strides=strides, activation=activation, name=name)
     return nn
 
 
 def csp_conv_downsample(inputs, filters, strides=2, activation="swish", name=""):
     # DownC: https://github.com/WongKinYiu/yolor/blob/paper/models/common.py#L584
-    max_down = keras.layers.MaxPooling2D(strides, strides=strides, padding="SAME")(inputs)
+    channel_axis = -1 if image_data_format() == "channels_last" else 1
+    max_down = layers.MaxPool2D(strides, strides=strides, padding="SAME")(inputs)
     max_down = conv_dw_pw_block(max_down, filters // 2, activation=activation, name=name + "max_down_")
-    conv_down = conv_dw_pw_block(inputs, inputs.shape[-1], activation=activation, name=name + "conv_down_1_")
+    conv_down = conv_dw_pw_block(inputs, inputs.shape[channel_axis], activation=activation, name=name + "conv_down_1_")
     conv_down = conv_dw_pw_block(conv_down, filters // 2, kernel_size=3, strides=strides, activation=activation, name=name + "conv_down_2_")
-    return tf.concat([conv_down, max_down], axis=-1)
+    return functional.concat([conv_down, max_down], axis=channel_axis)
 
 
 def CSPDarknet(
@@ -141,7 +146,7 @@ def CSPDarknet(
     model_name="",
 ):
     # base_channels = int(width_mul * 64)
-    inputs = keras.layers.Input(input_shape)
+    inputs = layers.Input(backend.align_input_shape_by_image_data_format(input_shape))
 
     """ Stem """
     stem_width = stem_width if stem_width > 0 else channels[0] // 2
@@ -170,7 +175,7 @@ def CSPDarknet(
         features.append(nn)
 
     nn = [features[ii] for ii in out_features]
-    model = keras.models.Model(inputs, nn, name=model_name)
+    model = models.Model(inputs, nn, name=model_name)
     return model
 
 
@@ -179,25 +184,28 @@ def CSPDarknet(
 
 def upsample_merge(inputs, csp_depth, use_shortcut_bn=True, activation="swish", name=""):
     # print(f">>>> upsample_merge inputs: {[ii.shape for ii in inputs] = }")
-    upsample = conv_dw_pw_block(inputs[-1], inputs[0].shape[-1], activation=activation, name=name + "up_")
+    channel_axis = -1 if image_data_format() == "channels_last" else 1
+    upsample = conv_dw_pw_block(inputs[-1], inputs[0].shape[channel_axis], activation=activation, name=name + "up_")
 
-    # inputs[0] = keras.layers.UpSampling2D(size=(2, 2), interpolation="nearest", name=name + "up")(fpn_out)
-    inputs[-1] = tf.image.resize(upsample, tf.shape(inputs[0])[1:-1], method="nearest")
-    nn = tf.concat(inputs, axis=-1)
+    # inputs[0] = layers.UpSampling2D(size=(2, 2), interpolation="nearest", name=name + "up")(fpn_out)
+    size = functional.shape(inputs[0])[1:-1] if image_data_format() == "channels_last" else functional.shape(inputs[0])[2:]
+    inputs[-1] = functional.resize(upsample, size, method="nearest")
+    nn = functional.concat(inputs, axis=channel_axis)
     use_shortcut, use_pre, use_post = False, True, False
-    nn = csp_stack(nn, csp_depth, nn.shape[-1] // 2, 1.0, use_shortcut, use_pre, use_post, use_shortcut_bn, activation=activation, name=name)
+    nn = csp_stack(nn, csp_depth, nn.shape[channel_axis] // 2, 1.0, use_shortcut, use_pre, use_post, use_shortcut_bn, activation=activation, name=name)
     return nn
 
 
 def downsample_merge(inputs, csp_depth, use_csp_downsample=False, use_shortcut_bn=True, activation="swish", name=""):
     # print(f">>>> downsample_merge inputs: {[ii.shape for ii in inputs] = }")
+    channel_axis = -1 if image_data_format() == "channels_last" else 1
     if use_csp_downsample:
-        inputs[0] = csp_conv_downsample(inputs[0], inputs[-1].shape[-1], activation=activation, name=name)
+        inputs[0] = csp_conv_downsample(inputs[0], inputs[-1].shape[channel_axis], activation=activation, name=name)
     else:
-        inputs[0] = conv_dw_pw_block(inputs[0], inputs[-1].shape[-1], 3, 2, activation=activation, name=name + "down_")
-    nn = tf.concat(inputs, axis=-1)
+        inputs[0] = conv_dw_pw_block(inputs[0], inputs[-1].shape[channel_axis], 3, 2, activation=activation, name=name + "down_")
+    nn = functional.concat(inputs, axis=channel_axis)
     use_shortcut, use_pre, use_post = False, True, False
-    nn = csp_stack(nn, csp_depth, nn.shape[-1] // 2, 1.0, use_shortcut, use_pre, use_post, use_shortcut_bn, activation=activation, name=name)
+    nn = csp_stack(nn, csp_depth, nn.shape[channel_axis] // 2, 1.0, use_shortcut, use_pre, use_post, use_shortcut_bn, activation=activation, name=name)
     return nn
 
 
@@ -208,12 +216,13 @@ def path_aggregation_fpn(features, fpn_depth=2, use_csp_downsample=False, use_sh
     #       ↓ [up -> concat]  ↑ [down -> concat]
     # p3 -> p3p4p5 ------------┴-> out0
     # features: [p3, p4, p5]
+    channel_axis = -1 if image_data_format() == "channels_last" else 1
     upsamples = [features[-1]]
     p_name = "p{}_".format(len(features) + 2)
     # upsamples: [p5], features[:-1][::-1]: [p4, p3] -> [p5, p4p5, p3p4p5]
     for id, ii in enumerate(features[:-1][::-1]):
         cur_p_name = "p{}".format(len(features) + 1 - id)
-        nn = conv_dw_pw_block(ii, ii.shape[-1] // 2, kernel_size=1, activation=activation, name=name + cur_p_name + "_down_")
+        nn = conv_dw_pw_block(ii, ii.shape[channel_axis] // 2, kernel_size=1, activation=activation, name=name + cur_p_name + "_down_")
         p_name = cur_p_name + p_name
         nn = upsample_merge([nn, upsamples[-1]], fpn_depth, use_shortcut_bn, activation=activation, name=name + p_name)
         upsamples.append(nn)
@@ -231,29 +240,32 @@ def path_aggregation_fpn(features, fpn_depth=2, use_csp_downsample=False, use_sh
 
 
 def yolor_head_single(inputs, filters, num_classes=80, num_anchors=3, use_object_scores=True, activation="swish", name=""):
-    initializer = tf.initializers.truncated_normal(stddev=0.2)
+    channel_axis = -1 if image_data_format() == "channels_last" else 1
+    initializer = initializers.truncated_normal(stddev=0.2)
 
     nn = conv_dw_pw_block(inputs, filters, 3, activation=activation, name=name + "1_")
-    nn = BiasLayer(initializer=initializer, name=name + "shift_channel")(nn)
+    nn = BiasLayer(initializer=initializer, axis=channel_axis, name=name + "shift_channel")(nn)
 
     ouput_classes = num_classes + (5 if use_object_scores else 4)  # num_anchors = 3, num_anchors * (80 + 5) = 255
-    nn = keras.layers.Conv2D(ouput_classes * num_anchors, kernel_size=1, name=name + "2_conv")(nn)
-    control_channels_layer = ChannelAffine(use_bias=False, name=name + "control_channel")
+    nn = layers.Conv2D(ouput_classes * num_anchors, kernel_size=1, name=name + "2_conv")(nn)
+    nn = nn if image_data_format() == "channels_last" else layers.Permute([2, 3, 1])(nn)
+    control_channels_layer = ChannelAffine(use_bias=False, axis=-1, name=name + "control_channel")
     control_channels_layer.ww_init = initializer
     nn = control_channels_layer(nn)
     # return nn
-    return keras.layers.Reshape([-1, ouput_classes], name=name + "output_reshape")(nn)
+    return layers.Reshape([-1, ouput_classes], name=name + "output_reshape")(nn)
 
 
 def yolor_head(inputs, num_classes=80, num_anchors=1, use_object_scores=True, activation="swish", classifier_activation="sigmoid", name=""):
+    channel_axis = -1 if image_data_format() == "channels_last" else 1
     outputs = []
     for id, input in enumerate(inputs):
         cur_name = name + "{}_".format(id + 1)
-        filters = int(input.shape[-1] * 2)
+        filters = int(input.shape[channel_axis] * 2)
         out = yolor_head_single(input, filters, num_classes, num_anchors, use_object_scores, activation=activation, name=cur_name)
         outputs.append(out)
     # return outputs
-    outputs = tf.concat(outputs, axis=1)
+    outputs = functional.concat(outputs, axis=1)
     return activation_by_name(outputs, classifier_activation, name="classifier_")
 
 
@@ -288,6 +300,10 @@ def YOLOR(
     rescale_mode="raw01",  # For decode predictions, raw01 means input value in range [0, 1].
     kwargs=None,  # Not using, recieving parameter
 ):
+    # Regard input_shape as force using original shape if first element is None or -1,
+    # else assume channel dimention is the one with min value in input_shape, and put it first or last regarding image_data_format
+    input_shape = backend.align_input_shape_by_image_data_format(input_shape)
+
     if backbone is None:
         # Save line width...
         csp_kwargs = {"use_pre": csp_use_pre, "use_post": csp_use_post, "input_shape": input_shape, "activation": activation, "model_name": "darknet"}
@@ -301,8 +317,8 @@ def YOLOR(
         else:
             features = model_surgery.get_pyramide_feature_layers(backbone)
             features = [features[id] for id in features_pick]
-        print(">>>> features:", {ii.name: ii.output_shape for ii in features})
-        features = [ii.output for ii in features]
+        feature_names, features = model_surgery.align_pyramide_feature_output_by_image_data_format(features)
+        print(">>>> features:", {ii: jj.shape for ii, jj in zip(feature_names, features)})
 
     backbone.trainable = False if freeze_backbone else True
     use_object_scores, num_anchors, anchor_scale = anchors_func.get_anchors_mode_parameters(anchors_mode, use_object_scores, num_anchors, anchor_scale)
@@ -310,8 +326,8 @@ def YOLOR(
 
     fpn_features = path_aggregation_fpn(features, fpn_depth, use_csp_downsample, use_shortcut_bn, activation=activation, name="pafpn_")
     outputs = yolor_head(fpn_features, num_classes, num_anchors, use_object_scores, activation, classifier_activation, name="head_")
-    outputs = keras.layers.Activation("linear", dtype="float32", name="outputs_fp32")(outputs)
-    model = keras.models.Model(inputs, outputs, name=model_name)
+    outputs = layers.Activation("linear", dtype="float32", name="outputs_fp32")(outputs)
+    model = models.Model(inputs, outputs, name=model_name)
     reload_model_weights(model, PRETRAINED_DICT, "yolor", pretrained)
 
     pyramid_levels = [pyramid_levels_min, pyramid_levels_min + len(features_pick) - 1]  # -> [3, 5]
